/*
 * Copyright 2004-2008 Analog Devices Inc.
 *
 * Licensed under the GPL-2 or later.
 */

#ifndef _BFIN_SEGMENT_H
#define _BFIN_SEGMENT_H

#define KERNEL_DS   (0x5)
#define USER_DS     (0x1)

#endif				/* _BFIN_SEGMENT_H */
