/*
 * arch/arm/mach-spear3xx/include/mach/entry-macro.S
 *
 * Low-level IRQ helper macros for SPEAr3xx machine family
 *
 * Copyright (C) 2009 ST Microelectronics
 * Viresh Kumar<viresh.kumar@st.com>
 *
 * This file is licensed under the terms of the GNU General Public
 * License version 2. This program is licensed "as is" without any
 * warranty of any kind, whether express or implied.
 */

		.macro	disable_fiq
		.endm

		.macro	arch_ret_to_user, tmp1, tmp2
		.endm
