/*
 * entry-macro.S
 *
 * Copyright (c) 2011 Picochip Ltd., Jamie Iles
 *
 * Low-level IRQ helper macros for picoXcell platforms
 *
 * This file is licensed under  the terms of the GNU General Public
 * License version 2. This program is licensed "as is" without any
 * warranty of any kind, whether express or implied.
 */
	.macro  disable_fiq
	.endm

	.macro  arch_ret_to_user, tmp1, tmp2
	.endm
