/* arch/arm/mach-zynq/include/mach/timex.h
 *
 *  Copyright (C) 2011 Xilinx
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __MACH_TIMEX_H__
#define __MACH_TIMEX_H__

/* the following is needed for the system to build but will be removed
   in the future, the value is not important but won't hurt
*/
#define CLOCK_TICK_RATE	(100 * HZ)

#endif
